// -*- mode: c++ -*-
// $Id: matrix.g++,v 1.3 2001/06/20 03:20:02 doug Exp $
// http://www.bagley.org/~doug/shootout/

import timer;

const int SIZE = 30;

int[][] mkmatrix(int rows, int cols) {
    int i, j, count = 1;
    int[][] m;
    m.length = rows;
    for (i=0; i<rows; i++) {
        m[i].length = cols;
        for (j=0; j<cols; j++) {
            m[i][j] = count++;
        }
    }
    return(m);
}

int[][] mmult(int rows, int cols, int[][] m1, int[][] m2, int[][] m3) {
    int i, j, k, val;
    for (i=0; i<rows; i++) {
        for (j=0; j<cols; j++) {
            val = 0;
            for (k=0; k<cols; k++) {
                val += m1[i][k] * m2[k][j];
            }
            m3[i][j] = val;
        }
    }
    return(m3);
}

int main(char[][] args) {
    int i, n = (args.length == 2) ? std.string.atoi(args[1]) : 1;
    
    Timer t = new Timer;
    int[][] m1 = mkmatrix(SIZE, SIZE);
    int[][] m2 = mkmatrix(SIZE,SIZE);
    //** Using dup here causes incorrect results.
    //** It appears that 'm3[i][j] = val;' above is not being evaluated correctly (??).
    //** If dup is used and the inner-most loop above is commented out, the expected results of all '0's is produced..
    //** Also, on the same machine (Windows - this is Linux), this code runs about 70% slower than the
    //**  equivalent C++ code produced with dmc [The Win and Linux D versions run at the same speed, but I don't have
    //** obj2asm available on Windows).
    //int[][] mm = m1.dup;
    int[][] mm = mkmatrix(SIZE,SIZE);

    for (i=0; i<n; i++) {
        mmult(SIZE, SIZE, m1, m2, mm);
    }

    printf("%d %d %d %d\n",mm[0][0],mm[2][3],mm[3][2],mm[4][4]);
    printf("%s\n",cast(char*)t.Elapsed());

    return(0);
}

//**  P4 2.2 Ghz, 512 MB RAM
//**  Intel v8.0    270165 1061760 1453695 1856025  5.87308 seconds (-O3 -mcpu=pentium4 -static)
//**  g++   v3.4.1  270165 1061760 1453695 1856025  6.68017 seconds (-O3 -fomit-stack-pointer -funroll-loops -mtune=pentium4 -static)
//**   g++  v3.4.1  270165 1061760 1453695 1856025  7.12449 seconds (-O3 -fomit-frame-pointer -mtune=pentium4 -static matrix.cpp -o matrix)
//**  DMC   v8.40   270165 1061760 1453695 1856025  9.15400 seconds (-o)
//**  DMD   v0.98   270165 1061760 1453695 1856025 15.67400 seconds (-O -inline -release)
